@@ -209,8 +209,8 @@ typedef struct r_cons_palette_t {
 	int rainbow_sz; // size of rainbow
 } RConsPalette;
 
-R_API const char *r_cons_rainbow_get(int idx, int last, bool bg);
-R_API void r_cons_rainbow_free();
+R_API char *r_cons_rainbow_get(int idx, int last, bool bg);
+R_API void r_cons_rainbow_free(void);
 R_API void r_cons_rainbow_new(int sz);
 
 typedef void (*RConsEvent)(void *);
@@ -249,7 +249,11 @@ typedef struct r_cons_canvas_t {
 #define RUNECODE_CORNER_TL 0xcf
 #define RUNECODE_CORNER_TR 0xd0
 #define RUNECODE_LINE_UP 0xd1
-#define RUNECODE_MAX 0xd2
+#define RUNECODE_CURVE_CORNER_TL 0xd2
+#define RUNECODE_CURVE_CORNER_TR 0xd3
+#define RUNECODE_CURVE_CORNER_BR 0xd4
+#define RUNECODE_CURVE_CORNER_BL 0xd5
+#define RUNECODE_MAX 0xd6
 
 #define RUNECODESTR_MIN 0xc8 // 200
 #define RUNECODESTR_LINE_VERT "\xc8"
@@ -262,7 +266,11 @@ typedef struct r_cons_canvas_t {
 #define RUNECODESTR_CORNER_TL "\xcf"
 #define RUNECODESTR_CORNER_TR "\xd0"
 #define RUNECODESTR_LINE_UP "\xd1"
-#define RUNECODESTR_MAX 0xd1
+#define RUNECODESTR_CURVE_CORNER_TL "\xd2"
+#define RUNECODESTR_CURVE_CORNER_TR "\xd3"
+#define RUNECODESTR_CURVE_CORNER_BR "\xd4"
+#define RUNECODESTR_CURVE_CORNER_BL "\xd5"
+#define RUNECODESTR_MAX 0xd5
 
 #define RUNE_LINE_VERT "│"
 #define RUNE_LINE_CROSS "┼" /* ├ */
@@ -274,6 +282,10 @@ typedef struct r_cons_canvas_t {
 #define RUNE_CORNER_TR "┐"
 #define RUNE_ARROW_RIGHT ">"
 #define RUNE_ARROW_LEFT "<"
+#define RUNE_CURVE_CORNER_TL "╭"
+#define RUNE_CURVE_CORNER_TR "╮"
+#define RUNE_CURVE_CORNER_BR "╯"
+#define RUNE_CURVE_CORNER_BL "╰"
 
 typedef char *(*RConsEditorCallback)(void *core, const char *file, const char *str);
 typedef int (*RConsClickCallback)(void *core, int x, int y);
@@ -339,6 +351,7 @@ typedef struct r_cons_t {
 #endif
 	bool flush;
 	bool use_utf8; // use utf8 features
+	bool use_utf8_curvy; // use utf8 curved corners
 	int linesleep;
 	int pagesize;
 	char *break_word;
@@ -347,6 +360,7 @@ typedef struct r_cons_t {
 	bool use_color;
 	bool use_tts;
 	bool filter;
+	char* (*rgbstr)(char *str, ut64 addr);
 } RCons;
 
 // XXX THIS MUST BE A SINGLETON AND WRAPPED INTO RCons */
@@ -448,6 +462,8 @@ enum {
 	LINE_TRUE,
 	LINE_FALSE,
 	LINE_UNCJMP,
+	LINE_NOSYM_VERT,
+	LINE_NOSYM_HORIZ
 };
 
 typedef struct r_cons_canvas_line_style_t {
@@ -499,6 +515,8 @@ R_API void r_cons_canvas_line_diagonal(RConsCanvas *c, int x, int y, int x2, int
 R_API void r_cons_canvas_line_square(RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style);
 R_API int r_cons_canvas_resize(RConsCanvas *c, int w, int h);
 R_API void r_cons_canvas_fill(RConsCanvas *c, int x, int y, int w, int h, char ch, int replace);
+R_API void r_cons_canvas_line_square_defined (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int bendpoint, int isvert);
+R_API void r_cons_canvas_line_back_edge (RConsCanvas *c, int x, int y, int x2, int y2, RCanvasLineStyle *style, int ybendpoint1, int xbendpoint, int ybendpoint2, int isvert);
 
 R_API RCons *r_cons_new(void);
 R_API RCons *r_cons_singleton(void);
@@ -565,6 +583,7 @@ R_API void r_cons_memset(char ch, int len);
 R_API void r_cons_visual_flush(void);
 R_API void r_cons_visual_write(char *buffer);
 R_API int r_cons_is_utf8(void);
+R_API void r_cons_cmd_help(const char * help[], bool use_color);
 
 /* input */
 //R_API int  r_cons_fgets(char *buf, int len, int argc, const char **argv);
@@ -575,7 +594,7 @@ R_API int r_cons_eof(void);
 
 R_API int r_cons_palette_init(const unsigned char *pal);
 R_API int r_cons_pal_set(const char *key, const char *val);
-R_API void r_cons_pal_update_event();
+R_API void r_cons_pal_update_event(void);
 R_API void r_cons_pal_free(void);
 R_API void r_cons_pal_init(const char *foo);
 R_API char *r_cons_pal_parse(const char *str);
@@ -592,6 +611,7 @@ R_API bool r_cons_isatty(void);
 R_API int r_cons_get_cursor(int *rows);
 R_API int r_cons_arrow_to_hjkl(int ch);
 R_API int r_cons_html_print(const char *ptr);
+R_API char *r_cons_html_filter(const char *ptr, int *newlen);
 
 // TODO: use gets() .. MUST BE DEPRECATED
 R_API int r_cons_fgets(char *buf, int len, int argc, const char **argv);
@@ -610,6 +630,7 @@ R_API void r_cons_rgb(ut8 r, ut8 g, ut8 b, int is_bg);
 R_API void r_cons_rgb_fgbg(ut8 r, ut8 g, ut8 b, ut8 R, ut8 G, ut8 B);
 R_API void r_cons_rgb_init(void);
 R_API char *r_cons_rgb_str(char *outstr, ut8 r, ut8 g, ut8 b, int is_bg);
+R_API char *r_cons_rgb_str_off(char *outstr, ut64 off);
 R_API void r_cons_color(int fg, int r, int g, int b);
 R_API char *r_cons_color_random(int bg);
 R_API char *r_cons_color_random_string(int bg);
@@ -712,6 +733,8 @@ typedef struct r_ascii_node_t {
 	int h;
 
 	int layer;
+	int layer_height;
+	int layer_width;
 	int pos_in_layer;
 	int is_dummy;
 	int is_reversed;
